from datetime import datetime

from discord import (Activity, ActivityType, Embed, Game, HTTPException,
                     Status, Streaming)
from discord.ext import commands
from rich.traceback import install as install_traceback

from discordbot.botmodules import apis, serverdata

install_traceback()
from rich.pretty import install as install_pretty

install_pretty()

#

extensionfolder = "discordbot.botcmds"
extensions = ['basic','support','moderation','games','help','channels','music','owneronly','converters','embedgenerator']
all_prefixes = ["/","!","$",".","-",">","?"]

# Own functions



# Own classes

class MyContext(commands.Context):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.apis = apis

        if self.guild is not None:
            self.data = serverdata.Server.getServer(self.guild.id)

            self.database = serverdata.DjangoConnection(self.author, self.guild)

    async def sendEmbed(self, *args, message:str="", **kwargs):
        return await self.send(message, embed=self.getEmbed(*args, **kwargs))

    def getEmbed(self, title:str, description:str="", color:int=0x000000, fields:list=[], inline=True, thumbnailurl:str=None, authorurl:str="", authorname:str=None, footertext:str="Angefordert von USER", footerurl:str="AVATARURL", timestamp=False):
        EMBED = Embed(title=title, description=description, color=color)
        EMBED.set_footer(text=footertext.replace("USER", str(self.author.name+"#"+self.author.discriminator)), icon_url=footerurl.replace("AVATARURL", str(self.author.avatar_url)))
        
        if timestamp:
            EMBED.timestamp = datetime.utcnow() if timestamp is True else timestamp
        for field in fields:
            EMBED.add_field(name=field[0], value=field[1], inline=bool(field[2] if len(field) > 2 else inline))
        if thumbnailurl:
            EMBED.set_thumbnail(url=thumbnailurl.strip())
        if authorname:
            if authorurl and ("https://" in authorurl or "http://" in authorurl):
                EMBED.set_author(name=authorname, url=authorurl.strip())
            else:
                EMBED.set_author(name=authorname)
        return EMBED

    async def tick(self, value=True):
        emoji = '\N{WHITE HEAVY CHECK MARK}' if value else '\N{CROSS MARK}'
        try:
            await self.message.add_reaction(emoji)
        except HTTPException:
            pass

    async def send_help(self):
        await self.invoke(self.bot.get_command("help"), self.invoked_with)


class MyBot(commands.Bot):
    def __init__(self, **kwargs):
        super().__init__(self.get_command_prefix, **kwargs)

    def get_command_prefix(self, client, message):
        if message.guild:
            prefixes = ['/']
        else:
            prefixes = all_prefixes
        return commands.when_mentioned_or(*prefixes)(client, message)
        
    async def get_context(self, message, *, cls=MyContext):
        return await super().get_context(message, cls=cls)

    def getEmbed(self, title: str, description: str = "", color: int = 0x000000, fields: list = [], inline=True, thumbnailurl: str = None, authorurl: str = "", authorname: str = None, footertext: str = None, footerurl: str = None, timestamp=False):
        EMBED = Embed(title=title, description=description, color=color)
        if footertext:
            if footerurl:
                EMBED.set_footer(text=footertext, icon_url=footerurl)
            else:
                EMBED.set_footer(text=footertext)

        if timestamp:
            EMBED.timestamp = datetime.utcnow() if timestamp is True else timestamp
        for field in fields:
            EMBED.add_field(name=field[0], value=field[1], inline=bool(
                field[2] if len(field) > 2 else inline))
        if thumbnailurl:
            EMBED.set_thumbnail(url=thumbnailurl.strip())
        if authorname:
            if authorurl and ("https://" in authorurl or "http://" in authorurl):
                EMBED.set_author(name=authorname, url=authorurl.strip())
            else:
                EMBED.set_author(name=authorname)
        return EMBED


# create Bot

bot = MyBot(
    description='Das ist eine Beschreibung!',
    case_insensitive=True,
    activity=Activity(type=ActivityType.listening, name="/help"),
    status=Status.idle,
    help_command=None,
)

# Events

from discordbot.botevents import setup

setup(bot)

@bot.event
async def on_ready():
    print(f"[Bot] - Logged in as '{bot.user.name}' - '{bot.user.id}'")
    for extension in extensions:
        try:
            bot.load_extension(extensionfolder+"."+extension)
        except commands.errors.ExtensionAlreadyLoaded:
            pass
    return

# Hidden commands

@bot.command(aliases=["."], hidden=True)
async def destroy(ctx):
    pass


# Start

def run(TOKEN):
    bot.run(TOKEN, bot=True, reconnect=True)

if __name__ == "__main__":
    print("[Bot] - You must run this bot via your manage.py file: python3.8 manage.py run-discorbot")



from discord.ext import commands
from discord import Embed, User, Member, opus, FFmpegPCMAudio, PCMVolumeTransformer, VoiceChannel
from fuzzywuzzy import process
import os

###

def chunks(lst, n):
    for i in range(0, len(lst), n):
        yield lst[i:i + n]

###

filespath = os.path.join(os.path.dirname(os.path.dirname(os.path.realpath(__file__))), "botfiles")
memespath = os.path.join(filespath, "memes")

# opus.load_opus('opus')

ffmpeg_options = {
    'options': '-vn',
    'executable': os.path.join(filespath,"ffmpeg.exe")
}

radios = {
    "swisspop": "http://www.radioswisspop.ch/live/mp3.m3u",
    "nrjbern":  "https://energybern.ice.infomaniak.ch/energybern-high.mp3",
}



#####

class Music(commands.Cog):
    def __init__(self, bot):
        self.bot = bot
        self.color = 0xee00ff

    @commands.Cog.listener()
    async def on_voice_state_update(self, member, before, after):
        if os.getenv("DEBUG", False):
            filespath = os.path.join(os.path.dirname(
                os.path.dirname(os.path.realpath(__file__))), "botfiles")
            memespath = os.path.join(filespath, "memes")

            ffmpeg_options = {
                'options': '-vn',
                'executable': os.path.join(filespath, "ffmpeg.exe")
            }

            # *Grillenzirpen* nach Streamende
            if before.channel and before.self_stream and not after.self_stream:
                voice_client = before.channel.guild.voice_client
                if voice_client is None:
                    voice_client = await before.channel.connect()
                elif voice_client.is_playing():
                    voice_client.stop()
                    await voice_client.move_to(before.channel)

                player = PCMVolumeTransformer(FFmpegPCMAudio(
                    source=os.path.join(memespath, "grillenzirpen.wav"), **ffmpeg_options))
                voice_client.play(player, after=lambda e: print(
                    '[Music] - Error: %s' % e) if e else None)


    # sich in Entwicklung befindende Befehle
    if os.getenv("DEBUG", False):
        @commands.command(
            name='memes',
            brief='Liste alle Memes auf',
            description='Liste alle Memes auf',
            aliases=[],
            help="Benutze /memes um eine Liste aller Memes zu erhalten.",
            usage=""
        )
        async def memes(self, ctx):
            filenames = list(os.listdir(memespath))
            chunklist = list(chunks(filenames, 25))
            for chunkindex in range(len(chunklist)):
                chunk = chunklist[chunkindex]
                await ctx.sendEmbed(
                    title="Memes (Seite "+str(chunkindex+1)+"/"+str(len(chunklist))+")",
                    color=self.color,
                    fields=[("Meme", filename.split(".")[0]) for filename in chunk]
                )


        @commands.command(
            name='meme',
            brief='Spiele Memes',
            description='Spiele Memes von einer Audiodatei!',
            aliases=[],
            help="Benutze /meme <Name> um einen Meme abzuspielen.",
            usage="<Suche>"
        )
        @commands.guild_only()
        async def meme(self, ctx, search:str="windows-xp-error", *args):
            search = " ".join([search]+args)
            filenames = list(os.listdir(memespath))

            result = process.extractOne(search, filenames)
            filename = result[0]

            print("[Music] - Suchergebnis:", search, result)

            if result[1] >= 75:
                player = PCMVolumeTransformer(FFmpegPCMAudio(source=os.path.join(memespath, filename), **ffmpeg_options))

                if ctx.voice_client.is_playing():
                    ctx.voice_client.stop()

                ctx.voice_client.play(player, after=lambda e: print('[Music] - Fehler: %s' % e) if e else None)
                await ctx.sendEmbed(title="Memetime!", color=self.color, fields=[("Meme",str(filename).split(".")[0])])
            else:
                raise commands.BadArgument(message="Es wurden keine mit '{}' übereinstimmende Audiodatei gefunden.".format(search))

        # from example

        @commands.command(
            name='play',
            brief='Spiele Musik',
            description='Spiele Musik von Youtube und anderen Plattformen!',
            aliases=["yt","youtube"],
            help="Benutze /play <Url/Suche> um einen Song abzuspielen.",
            usage="<Url/Suche>"
        )
        @commands.guild_only()
        async def play(self, ctx, search: str, *args):
            url = " ".join([search]+args)
            async with ctx.typing():
                player = list(await ctx.data.musicqueue.createYoutubePlayer(url, loop=self.bot.loop))[0]
                if ctx.voice_client.is_playing():
                    ctx.data.musicqueue.addPlayer(player)
                    await player.send(ctx, status="Song zur Playlist hinzugefügt!")
                else:
                    player.play(ctx)
                    await player.send(ctx)


        @commands.command(
            name='stream',
            brief='Streame einen Stream',
            description='Streame einen Stream von Twitch oder YouTube',
            aliases=[],
            help="Benutze /stream <Url/Suche> den einen Stream zu streamen.",
            usage="<Url/Suche>"
        )
        @commands.guild_only()
        async def stream(self, ctx, search: str, *args):
            url = " ".join([search]+args)
            if url in radios:
                url = radios[url]

            async with ctx.typing():
                player = list(await ctx.data.musicqueue.createYoutubePlayer(url, loop=self.bot.loop, stream=True))[0]

                if ctx.voice_client.is_playing():
                    ctx.voice_client.stop()

                player.play(ctx)
                await player.send(ctx, "Stream wird direkt wiedergegeben!")

        @commands.command(
            name='nowplaying',
            brief='Was läuft gerade?',
            description='Sieh, was gerade läuft',
            aliases=["np"],
            help="Benutze /np um zu sehen, was aktuell läuft.",
            usage=""
        )
        @commands.guild_only()
        async def nowplaying(self, ctx):
            await ctx.data.musicqueue.sendNowPlaying(ctx)

        @commands.command(
            name='pause',
            brief='Pausiere Musik',
            description='Pausiere die aktuelle Musik',
            aliases=[],
            help="Benutze /pause um die aktuelle Musik zu pausieren.",
            usage=""
        )
        @commands.guild_only()
        async def pause(self, ctx):
            if ctx.voice_client and ctx.voice_client.is_playing():
                ctx.voice_client.pause()
                await ctx.sendEmbed(title="Musik pausiert", color=self.color)

        @commands.command(
            name='resume',
            brief='Führe Musik fort',
            description='Hebe die Pausierung der aktuellen Musik auf.',
            aliases=[],
            help="Benutze /resume um die aktuelle Musik weiter zu spielen.",
            usage=""
        )
        @commands.guild_only()
        async def resume(self, ctx):
            if ctx.voice_client and ctx.voice_client.is_paused():
                ctx.voice_client.resume()
                await ctx.sendEmbed(title="Pausierung aufgehoben", color=self.color)

        @commands.command(
            name='skip',
            brief='Überspringe Musik',
            description='Überspringe aktuelle Musik',
            aliases=[],
            help="Benutze /skip um den aktuellen Song zu überspringen.",
            usage="<Url/Suche>"
        )
        @commands.guild_only()
        async def skip(self, ctx):
            async with ctx.typing():
                if ctx.voice_client is not None and ctx.voice_client.is_playing():
                    ctx.voice_client.stop()
                if ctx.voice_client is not None:
                    ctx.data.musicqueue.playNext(ctx)
                    await ctx.data.musicqueue.sendNowPlaying(ctx)
                else:
                    raise commands.CommandError(message="Es wurde gar kein Song abgespielt.")

        @commands.command(
            name='volume',
            brief='Ändere die Lautstärke',
            description='Ändere die Lautstärke des Bots',
            aliases=["vol"],
            help="Benutze /volume <1-200> um die Lautstärke des Bots zu ändern.",
            usage="<1-200>"
        )
        @commands.guild_only()
        async def volume(self, ctx, newvolume: float):
            if ctx.voice_client is None:
                raise commands.CommandError("Der Bot ist nicht mit einem Sprachkanal verbunden.")
            elif not ctx.voice_client.source:
                raise commands.CommandError("Der Bot scheint aktuell nichts abzuspielen.")

            oldvolume = ctx.voice_client.source.volume * 100
            ctx.voice_client.source.volume = newvolume / 100

            await ctx.sendEmbed(title="Lautstärke geändert", color=self.color, fields=[("Zuvor", str(oldvolume)+"%"),("Jetzt",str(newvolume)+"%")])

        @commands.command(
            name='stop',
            brief='Stoppe Musik',
            description='Stoppe Musik!',
            aliases=["die","leave","disconnect"],
            help="Benutze /stop um den Bot aus dem Sprachkanal zu entfernen.",
            usage=""
        )
        @commands.guild_only()
        async def stop(self, ctx):
            if ctx.voice_client:
                await ctx.voice_client.disconnect()
            else:
                raise commands.CommandError(message="Der Bot war in gar keinem Sprachkanal!")

        @meme.before_invoke
        @play.before_invoke
        @stream.before_invoke
        @nowplaying.before_invoke
        @skip.before_invoke
        @volume.before_invoke
        async def autojoin(self, ctx):
            if ctx.author.voice and ctx.author.voice.channel.guild == ctx.guild:
                if ctx.voice_client is None:
                    await ctx.author.voice.channel.connect()
                #elif ctx.voice_client.is_playing():
                    #ctx.voice_client.stop()
                await ctx.voice_client.move_to(ctx.author.voice.channel)
            else:
                raise commands.CommandError("Du bist mit keinem Sprachkanal in diesem Server verbunden!")



    # Generelle Commands

    @commands.command(
        name='usersong',
        brief='Stalke musikhörende Leute',
        description='Erhalte Links zu dem Song, welcher jemand gerade hört',
        aliases=[],
        help="Benutze /usersong <Member> um den Song zu erhalten",
        usage="<Member>"
    )
    @commands.guild_only()
    async def usersong(self, ctx, Member:Member):
        found = False
        for activity in Member.activities:
            if str(activity.type) == "ActivityType.listening":
                try:
                    await ctx.sendEmbed(title="User Song", color=self.color, fields=[("Titel", activity.title),("Künstler", activity.artist),("Link", ("[Spotify](https://open.spotify.com/track/"+activity.track_id+")"))])
                except AttributeError:
                    raise commands.BadArgument(message="Scheinbar hört dieser Benutzer keinen richtigen Song.")
                found = True
        if not found:
            raise commands.BadArgument(message="Dieser Benutzer hört keinen Song!")




def setup(bot):
    bot.add_cog(Music(bot))



from discord.ext import commands
from discord import Game, Streaming, Activity, ActivityType, Status, Member, User, TextChannel, VoiceChannel, Role, Invite, Game, Emoji, PartialEmoji, Colour

from discordbot.bot import extensions, extensionfolder

CONVERTERS = {
    Member: commands.MemberConverter,
    User: commands.UserConverter,
    TextChannel: commands.TextChannelConverter,
    VoiceChannel: commands.VoiceChannelConverter,
    Role: commands.RoleConverter,
    Invite: commands.InviteConverter,
    Game: commands.GameConverter,
    Emoji: commands.EmojiConverter,
    PartialEmoji: commands.PartialEmojiConverter,
    Colour: commands.ColourConverter
}

class Owneronly(commands.Cog):
    def __init__(self, bot):
        self.bot = bot
        self.color = 0x000000


    @commands.command(
        brief="Lade die Bot-Extensionen neu",
    )
    @commands.is_owner()
    async def reload(self, ctx, extension:str=None):
        msg = await ctx.sendEmbed(title="Reload", color=self.color, fields=[("Status", "Reloading")])
        EMBED = ctx.getEmbed(title="Reload", color=self.color, fields=[])
        if extension in extensions:
            print("[Bot] - Reloading"+extension+"...")
            try:
                self.bot.unload_extension(extensionfolder+"."+extension)
            except:
                pass
            try:
                self.bot.load_extension(extensionfolder+"."+extension)
            except commands.errors.ExtensionAlreadyLoaded:
                pass
            EMBED.add_field(name="Status",value="Reloaded category "+extension.upper()+"!")
        else:
            print("[Bot] - Reloading all extensions...")
            for extension in extensions:
                try:
                    self.bot.unload_extension(extensionfolder+"."+extension)
                except:
                    pass
                try:
                    self.bot.load_extension(extensionfolder+"."+extension)
                except commands.errors.ExtensionAlreadyLoaded:
                    pass
            EMBED.add_field(name="Status",value="Reloaded all categories!")
        await msg.edit(embed=EMBED)
        print("[Bot] - Reload completed!")


    @commands.command(
        brief="Stoppe den Bot",
    )
    @commands.is_owner()
    async def stopbot(self, ctx):
        await ctx.sendEmbed(title="Stop", color=self.color, fields=[("Status", "Gestoppt")])
        await self.bot.logout()


    @commands.command(
        brief="Ändere den Status des Bots",
        usage="<Status> [Aktivität [Argumente(e)]]",
    )
    @commands.is_owner()
    async def status(self, ctx, STATUS:str="", ACTIVITY:str="", arg1:str="", *args):
        arg2 = " ".join(args)
        status = None
        activity = None
        if STATUS.lower() in ["on","online","green"]:
            status = Status.online
        elif STATUS.lower() in ["off","offline","invisible","grey"]:
            status = Status.invisible
        elif STATUS.lower() in ["dnd","donotdisturb","do_not_disturb","bittenichtstören","red"]:
            status = Status.dnd
        elif STATUS.lower() in ["idle","abwesend","orange","yellow"]:
            status = Status.idle

        if ACTIVITY.lower() in ["playing","spielt","game","play"]:
            activity=Game(name=arg1+" "+arg2)
        elif ACTIVITY.lower() in ["streaming","streamt","stream","live","twitch"]:
            activity=Streaming(url="https://twitch.tv/"+arg1, name=arg2)
        elif ACTIVITY.lower() in ["listening","listen","hört","hören","song"]:
            activity=Activity(type=ActivityType.listening, name=arg1+" "+arg2)
        elif ACTIVITY.lower() in ["watching","watch","schaut","video"]:
            activity=Activity(type=ActivityType.watching, name=arg1+" "+arg2)

        if status is not None or activity is not None:
            await ctx.bot.change_presence(status=status, activity=activity)
        else:
            await ctx.sendEmbed(title="Status ändern", color=0xff0000, inline=False,
            description="""
            **Syntax:**
            /status <STATUS> [AKTIVITÄT [ARGUMENT(E)]]

            **Mögliche Status:**
            on/online (Online)
            idle (Abwesend)
            dnd (Bitte nicht stören)
            off/offline (Unsichtbar)

            **Mögliche Aktivitäten:**
            spielt <SPIEL>
            streamt <TWITCH-NAME> <SPIEL>
            hört <SONG>
            schaut <VIDEO>
            """
            )

    @commands.command(
        brief="Führe einen Command als jemanden anderes aus",
        usage="<Member> <Command> [Argumente]",
        aliases=["runas"],
        help="Falls Unterbefehle verwendet werden, benutze bitte befehl_unterbefehl als Command",
    )
    @commands.is_owner()
    @commands.guild_only()
    async def sudo(self, ctx, member: Member, command: str, *args):
        _command = command.replace("_", " ")
        cmd = self.bot.get_command(_command)
        ctx.message.content = ctx.prefix+_command+ctx.message.content.split(command)[1]
        ctx.message.author = member
        ctx.author = member
        ctx.database = type(ctx.database)(ctx.author, ctx.guild)
        annotations = cmd.callback.__annotations__
        annotations.pop("return", None)
        arguments = list(args)
        for i, cls in enumerate(annotations.values()):
            if len(arguments) > i:
                if cls in CONVERTERS:
                    arguments[i] = await CONVERTERS[cls]().convert(ctx, arguments[i])
                else:
                    arguments[i] = cls(arguments[i])
        await ctx.invoke(cmd, *arguments)

def setup(bot):
    bot.add_cog(Owneronly(bot))



from discord.ext import commands
from discord import Embed, User, TextChannel, utils
from datetime import datetime as d
import typing

REGELN = {
    "1) Sei anständig":
        [
            "Sei nett zu anderen Leuten und behandle sie so, wie auch du behandelt werden möchtest!"
        ],
    "2) Text":
        [
            "Spamming ist verboten!","Werbung ist verboten!"
        ],
    "3) NSFW":
        [
            "Anstössige Inhalte werden sofort gelöscht und der Autor mit einem Bann bestraft!","Hier sind auch Kinder und Jugendliche auf diesem Server!"
        ],
    "4) Sicherheit":
        [
            "Anweisungen von Moderatoren, Supportern und Admins müssen befolgt werden!","Falls jemand ohne Grund nach persönlichen Daten fragt, ignoriert bitte die Nachricht und meldet sie einem anderen Admin.","Sendet nie jemandem euer Passwort!"
        ],
    "5) Ton":
        [
            "Benutzt keinen Stimmverzerrer!", "Macht keine unnötigen Hintergrundgeräusche!"
        ],
    "6) Empfehlungen":
        [
            "Habt Spass!"
        ],
    "7) Sprachkanäle":
        [
            "Channel Hopping unterlassen"
        ],
}


class Basic(commands.Cog):
    def __init__(self, bot):
        self.bot = bot
        self.color = 0xffffff

    @commands.command(
        brief="Zeigt den Ping des Bots an",
        description='Gibt den aktuellen Ping zurück',
        aliases=['p'],
        help="Gib einfach /ping ein und warte ab.",
        usage=""
    )
    async def ping(self, ctx):
        start = d.timestamp(d.now())
        msg = await ctx.sendEmbed(title="Aktueller Ping", color=self.color, fields=[("Ping", "Berechnen...")])
        embed = ctx.getEmbed(title="Aktueller Ping", color=self.color, fields=[("Ping", str(int(( d.timestamp( d.now() ) - start ) * 1000))+"ms")])
        await msg.edit(embed=embed)
        return

    @commands.command(
        brief="Schreibt dir nach",
        description="Gibt den angegebenen Text zurück",
        aliases=["copy"],
        help="Benutze /say <Text> und der Bot schickt dir den Text zurück",
        usage="<Text>"
        )
    async def say(self, ctx, text:str, *args):
        txt = " ".join([text]+args)
        await ctx.send(txt)

    @commands.command(
        brief="Siehe den Avatar eines Benutzers",
        description="Erhalte den Standardavatar und Avatar eines Benutzers",
        aliases=["defaultavatar"],
        help="Benutze /avatar <User> und du siehst, welchen Avatar die Person hat",
        usage="<User>"
        )
    async def avatar(self, ctx, user:User):
        await ctx.sendEmbed(title="Avatar", fields=[("Benutzer",user.mention),("Standardavatar",user.default_avatar)], thumbnailurl=str(user.avatar_url))


    # @commands.command(
    #     brief="Spamt jemanden voll",
    #     description="Schickt jemandem ein paar Nachrichten",
    #     aliases=["troll"],
    #     help="Benutze /spam <User> und der Bot spamt den User voll",
    #     usage="<Kanal/Benutzer> [Anzahl<=10] [Text]"
    #     )
    # async def spam(self,ctx,what: typing.Union[TextChannel,User],anzahl:int=5,*args):
    #     anzahl = int(anzahl if anzahl <= 10 else 10)
    #     text = str(" ".join(str(i) for i in args))
    #     empty = not (len(text) > 0 and not text == (" "*len(text)))
    #     for i in range(anzahl):
    #         if not empty:
    #             await what.send(text+" von "+ctx.author.name+"#"+ctx.author.discriminator)
    #         else:
    #             await what.send("Spam"+" von "+ctx.author.name+"#"+ctx.author.discriminator)
    #     return


    @commands.command(
        brief="Zeigt die Regeln",
        description="Schickt die Regeln in den Chat",
        aliases=["rules"],
        help="Benutze /regeln um dich oder jemand anderes daran zu erinnern!",
        usage="<Kanal/Benutzer> [Anzahl<100] [Text]"
        )
    async def regeln(self,ctx):
        EMBED = Embed(title="Regeln", color=self.color, description="Das Nichtbeachten der Regeln kann mit einem Ban, Kick oder Mute bestraft werden!")
        owner = self.bot.get_user(self.bot.owner_id)
        if owner:
            EMBED.set_footer(text=f'Besitzer dieses Bots ist {owner.name}#{owner.discriminator}',icon_url=owner.avatar_url)
        for kategorie in REGELN:
            EMBED.add_field(name=kategorie,value="```nimrod\n- "+ ("\n- ".join(regel for regel in REGELN[kategorie])) +"```",inline=False)
        await ctx.send(embed=EMBED)


    @commands.command(
        brief="Erhalte eine Einladung",
        description="Schickt dir eine Einladung zum Server und Bot",
        aliases=["invitelink"],
        help="Benutze /invite und erhalte eine Einladung zu diesem Server, dem Bot-Server und einen Link, um den Bot zum eigenen Server hinzuzufügen",
        usage=""
        )
    @commands.bot_has_permissions(manage_guild = True)
    async def invite(self,ctx):
        try:
            invite = await ctx.guild.vanity_invite()
        except:
            invite = utils.get(list(await ctx.guild.invites()), max_age=0, max_uses=0, temporary=False)
            if not invite:
                invite = await ctx.channel.create_invite()
        await ctx.sendEmbed(title="Einladungen", color=self.color, fields=[("Dieser Server", invite.url),("Bot Server","https://rebrand.ly/RUdiscord"),("Bot","https://rebrand.ly/RUdiscordbot")])

def setup(bot):
    bot.add_cog(Basic(bot))



# pylint: disable=no-member, unsubscriptable-object

from django.db import models
from django.conf import settings
from django.urls import reverse
from django.utils import timezone

from discordbot.botmodules.bots import VierGewinntBot

import time
import uuid

# Helper functions


def get_discordbot_domain():
    return getattr(settings, "DISCORDBOT_DOMAIN", None)


# Basic

class Member(models.Model):
    server = models.ForeignKey("Server", on_delete=models.CASCADE)
    user = models.ForeignKey("User", on_delete=models.CASCADE, related_name="servers")

    def __str__(self):
        return str(self.server.name)+" - "+str(self.user.name)
    __str__.short_description = "Mitglied"

    class Meta():
        verbose_name = "Mitglied"
        verbose_name_plural = "Mitglieder"

    objects = models.Manager()

class Server(models.Model):
    id = models.CharField("Discord ID", primary_key=True, max_length=20)
    name = models.CharField("Name", default="", blank=True, max_length=100)
    members = models.ManyToManyField("User", through="Member")

    def reportCount(self):
        return self.reports.count()
    reportCount.short_description = "Reports"

    def getReports(self, user=None):
        if user is None:
            reports = []
            for member in self.members.all():
                count = member.reportCount()
                if count > 0:
                    reports.append({
                        "name": str(count)+" Report(s)",
                        "value": member.mention,
                        "inline": False
                    })
            return reports
        else:
            return user.getReports(server=self)

    def memberCount(self):
        return self.members.count()

    def __str__(self):
        return self.name+" ("+str(self.id)+")"
    __str__.short_description = "Server"

    class Meta():
        verbose_name = "Server"
        verbose_name_plural = "Server"

    objects = models.Manager()

class User(models.Model):
    id = models.CharField("Discord ID", primary_key=True, max_length=20)
    name = models.CharField("Name", default="", blank=True, max_length=100)

    def reportCount(self):
        return self.reports.count()
    reportCount.short_description = "Reports"

    def createdReportCount(self):
        return self.created_reports.count()
    createdReportCount.short_description = "Erstellte Reports"

    def serverCount(self):
        return self.servers.count()
    serverCount.short_description = "Anzahl Server"

    def joinServer(self, server):
        if not server.members.filter(id=self.id).exists():
            server.members.add(self)

    def getReports(self, server=None):
        if server is None:
            reports = self.reports.all()
        else:
            reports = self.reports.filter(server=server)
        return [
            report.getEmbedField() for report in reports
        ]

    @property
    def mention(self):
        return "<@"+str(self.id)+">"

    def __str__(self):
        return self.name+" ("+str(self.id)+")"
    __str__.short_description = "User"

    class Meta():
        verbose_name = "Benutzer"
        verbose_name_plural = "Benutzer"
    
    objects = models.Manager()

# Support

class Report(models.Model):
    server = models.ForeignKey("Server", on_delete=models.CASCADE, related_name="reports")
    user = models.ForeignKey("User", on_delete=models.CASCADE, related_name="reports")
    reported_by = models.ForeignKey("User", on_delete=models.SET_NULL, null=True, related_name="created_reports", verbose_name="Gemeldet von")

    reason = models.CharField("Grund", default="", blank=True, max_length=250)
    timestamp = models.DateTimeField("Zeitpunkt", auto_now_add=True)

    def getEmbedField(self):
        return {
            "name": str(self.timestamp.strftime('%d.%m.%Y - %H:%M:%S')),
            "value": str(self.reason)+" - "+self.reported_by.mention,
            "inline": False
        }

    def __str__(self):
        return "Report #"+str(self.pk)
    __str__.short_description = "Report"

    class Meta():
        verbose_name = "Report"
        verbose_name_plural = "Reports"

    objects = models.Manager()

# Games

## AmongUs

AMONGUS_PLAYER_COLORS = {
    'red':      ("#C51111", (179, 17, 17),   ":heart:",                 '❤'),
    'blue':     ("#132FD2", (19, 47, 210),   ":blue_heart:",            '💙'),
    'green':    ("#127F2D", (18, 127, 45),   ":green_heart:",           '💚'),
    'pink':     ("#ED53B9", (237, 83, 185),  ":heartpulse:",            '💗'),
    'orange':   ("#EF7D0E", (239, 125, 14),  ":orange_heart:",          '🧡'),
    'yellow':   ("#F3F558", (243, 246, 88),  ":yellow_heart:",          '💛'),
    'black':    ("#3F484E", (63, 72, 78),    ":black_heart:",           '🖤'),
    'white':    ("#D5E0EF", (213, 224, 239), ":white_heart:",           '🤍'),
    'purple':   ("#6B30BC", (107, 48, 188),  ":purple_heart:",          '💜'),
    'brown':    ("#72491E", (114, 37, 30),   ":brown_heart:",           '🤎'),
    'cyan':     ("#39FEDB", (57, 254, 219),  ":regional_indicator_c:",  '🇨'),
    'lime':     ("#50EF3A", (80, 239, 58),   ":regional_indicator_l:",  '🇱'),
}

AMONGUS_EMOJI_COLORS = {v[3]: k for k, v in AMONGUS_PLAYER_COLORS.items()}

class AmongUsGame(models.Model):
    api_key = models.UUIDField("API Key", default=uuid.uuid4)

    creator = models.ForeignKey("User", on_delete=models.CASCADE)
    guild = models.ForeignKey("Server", on_delete=models.CASCADE)
    
    voice_channel_id = models.CharField("Voice channel ID", max_length=20)
    text_channel_id = models.CharField("Text channel ID", max_length=20)
    text_message_id = models.CharField("Text message ID", max_length=20, default=None, null=True)

    code = models.CharField("Gamecode", max_length=6, default="", blank=True)

    last_edited = models.DateTimeField("Last edited", auto_now=True)
    last_tracking_data = models.DateTimeField("Last tracking data", default=None, null=True)

    state_ingame = models.BooleanField("In game", default=False)
    state_meeting = models.BooleanField("Meeting ongoing", default=False)

    p_red_name = models.CharField("Red's name", max_length=50, default="", blank=True)
    p_red_alive = models.BooleanField("Red's alive", default=True)
    p_red_exists = models.BooleanField("Red exists", default=False)
    p_red_userid = models.CharField("Red's Discord ID", max_length=50, default="", blank=True)
    p_blue_name = models.CharField("Blue's name", max_length=50, default="", blank=True)
    p_blue_alive = models.BooleanField("Blue's alive", default=True)
    p_blue_exists = models.BooleanField("Blue exists", default=False)
    p_blue_userid = models.CharField("Blue's Discord ID", max_length=50, default="", blank=True)
    p_green_name = models.CharField("Green's name", max_length=50, default="", blank=True)
    p_green_alive = models.BooleanField("Green's alive", default=True)
    p_green_exists = models.BooleanField("Green exists", default=False)
    p_green_userid = models.CharField("Green's Discord ID", max_length=50, default="", blank=True)
    p_pink_name = models.CharField("Pink's name", max_length=50, default="", blank=True)
    p_pink_alive = models.BooleanField("Pink's alive", default=True)
    p_pink_exists = models.BooleanField("Pink exists", default=False)
    p_pink_userid = models.CharField("Pink's Discord ID", max_length=50, default="", blank=True)
    p_orange_name = models.CharField("Orange's name", max_length=50, default="", blank=True)
    p_orange_alive = models.BooleanField("Orange's alive", default=True)
    p_orange_exists = models.BooleanField("Orange exists", default=False)
    p_orange_userid = models.CharField("Orange's Discord ID", max_length=50, default="", blank=True)
    p_yellow_name = models.CharField("Yellow's name", max_length=50, default="", blank=True)
    p_yellow_alive = models.BooleanField("Yellow's alive", default=True)
    p_yellow_exists = models.BooleanField("Yellow exists", default=False)
    p_yellow_userid = models.CharField("Yellow's Discord ID", max_length=50, default="", blank=True)
    p_black_name = models.CharField("Black's name", max_length=50, default="", blank=True)
    p_black_alive = models.BooleanField("Black's alive", default=True)
    p_black_exists = models.BooleanField("Black exists", default=False)
    p_black_userid = models.CharField("Black's Discord ID", max_length=50, default="", blank=True)
    p_white_name = models.CharField("White's name", max_length=50, default="", blank=True)
    p_white_alive = models.BooleanField("White's alive", default=True)
    p_white_exists = models.BooleanField("White exists", default=False)
    p_white_userid = models.CharField("White's Discord ID", max_length=50, default="", blank=True)
    p_purple_name = models.CharField("Purple's name", max_length=50, default="", blank=True)
    p_purple_alive = models.BooleanField("Purple's alive", default=True)
    p_purple_exists = models.BooleanField("Purple exists", default=False)
    p_purple_userid = models.CharField("Purple's Discord ID", max_length=50, default="", blank=True)
    p_brown_name = models.CharField("Brown's name", max_length=50, default="", blank=True)
    p_brown_alive = models.BooleanField("Brown's alive", default=True)
    p_brown_exists = models.BooleanField("Brown exists", default=False)
    p_brown_userid = models.CharField("Brown's Discord ID", max_length=50, default="", blank=True)
    p_cyan_name = models.CharField("Cyan's name", max_length=50, default="", blank=True)
    p_cyan_alive = models.BooleanField("Cyan's alive", default=True)
    p_cyan_exists = models.BooleanField("Cyan exists", default=False)
    p_cyan_userid = models.CharField("Cyan's Discord ID", max_length=50, default="", blank=True)
    p_lime_name = models.CharField("Lime's name", max_length=50, default="", blank=True)
    p_lime_alive = models.BooleanField("Lime's alive", default=True)
    p_lime_exists = models.BooleanField("Lime exists", default=False)
    p_lime_userid = models.CharField("Lime's Discord ID", max_length=50, default="", blank=True)

    
    def get_tracker_url(self):
        domain = get_discordbot_domain()
        if domain:
            return domain+reverse("discordbot:amongus-tracker-post")
        else:
            return "Frage den Bot-Owner!"

    def reset(self):
        self.code = ""
        self.state_ingame = False
        self.state_meeting = False
        for c in AMONGUS_PLAYER_COLORS:
            setattr(self, f'p_{c}_name', "")
            setattr(self, f'p_{c}_exists', False)
            setattr(self, f'p_{c}_alive', True)

    def post_data(self, data:dict):
        if "api_key" in data and data["api_key"] == str(self.api_key):
            if "reset" in data and data["reset"]:
                self.reset()
            if "code" in data:
                self.code = str(data["code"])
            if "state" in data:
                if "ingame" in data["state"]:
                    self.state_ingame = bool(data["state"]["ingame"])
                if "meeting" in data["state"]:
                    self.state_meeting = bool(data["state"]["meeting"])
            if "players" in data:
                for c in data["players"]:
                    if c in AMONGUS_PLAYER_COLORS:
                        if "name" in data["players"][c]:
                            setattr(self, f'p_{c}_name', data["players"][c]["name"])
                        if "alive" in data["players"][c]:
                            setattr(self, f'p_{c}_alive', data["players"][c]["alive"])
                        if "exists" in data["players"][c]:
                            setattr(self, f'p_{c}_exists', data["players"][c]["exists"])
                            
            self.last_tracking_data = timezone.now()
            self.save()
            return {"success": True}
        else:
            return {"error": "invalid-api-key", "error_message": "API Key is missing or wrong!"}

    def get_data(self):
        players = {c: {
                "name": getattr(self, f'p_{c}_name'),
                "alive": getattr(self, f'p_{c}_alive'),
                "userid": getattr(self, f'p_{c}_userid'),
                "exists": getattr(self, f'p_{c}_exists'),
            } for c in AMONGUS_PLAYER_COLORS
        }

        return {
            "id": self.pk,
            "code": self.code,
            "state": {
                "ingame": self.state_ingame,
                "meeting": self.state_meeting,
            },
            "players": players,
            "last_edited": self.last_edited,
            "last_tracking_data": self.last_tracking_data,
        }

    # User

    def remove_user(self, userid, save=False):
        for c in AMONGUS_PLAYER_COLORS:
            if getattr(self, f'p_{c}_userid') == str(userid):
                setattr(self, f'p_{c}_userid', "")

        if save:
            self.save()

    def set_user(self, userid, color, save=False):
        self.remove_user(userid)
        setattr(self, f'p_{color}_userid', str(userid))

        if save:
            self.save()


    def __str__(self):
        return "AmongUs #"+str(self.pk)

    class Meta:
        verbose_name = 'AmongUs Game'
        verbose_name_plural = 'AmongUs Games'
        unique_together = ('guild', 'creator',)

    objects = models.Manager()


## VierGewinnt

VIERGEWINNT_PLAYER_EMOJIS = ["⬛", "🟥", "🟨"]
VIERGEWINNT_WALL_EMOJI = "🟦"
VIERGEWINNT_NUMBER_EMOJIS = ["1️⃣", "2️⃣", "3️⃣", "4️⃣", "5️⃣", "6️⃣", "7️⃣", "8️⃣", "9️⃣", "🔟"]

def VIERGEWINNT_DEFAULT_GAME():
    return [[0 for _ in range(7)] for _ in range(6)]

class VierGewinntGame(models.Model):
    width = models.PositiveSmallIntegerField("Width", default=7)
    height = models.PositiveSmallIntegerField("Height", default=6)

    game = models.JSONField("Game", default=VIERGEWINNT_DEFAULT_GAME)

    finished = models.BooleanField("Finished", default=False)

    current_player = models.PositiveSmallIntegerField("Current player", default=1)

    player_1_id = models.CharField("Player 1 ID", max_length=32)
    player_2_id = models.CharField("Player 2 ID", max_length=32, default=None, null=True)
    winner_id = models.CharField("Winner ID", max_length=32, default=None, null=True)

    channel_id = models.CharField("Channel ID", max_length=32)
    message_id = models.CharField("Message ID", max_length=32)

    time_created = models.DateTimeField("Time created", auto_now_add=True)
    time_edited = models.DateTimeField("Time edited", auto_now=True)

    # Create

    @classmethod
    def create(self, width:int=7, height:int=6, **kwargs):
        width = 10 if width > 10 else 4 if width < 4 else width
        height = 4 if height < 4 else 14 if height > 14 else height
        kwargs["game"] = [[0 for _ in range(width)] for _ in range(height)]
        return self.objects.create(width=width, height=height, **kwargs)

    # Show

    def _get_gameboard(self):
        W = VIERGEWINNT_WALL_EMOJI
        P = VIERGEWINNT_PLAYER_EMOJIS
        N = VIERGEWINNT_NUMBER_EMOJIS

        numberline = (W+"".join(N[:int(self.width)])+W+"\n")
        gamelines = "\n".join([(W+"".join([P[p] for p in row])+W) for row in self.rows])+"\n"
        
        return numberline + gamelines + numberline

    def _get_players(self):
        pl = []
        for i in range(1, 2+1):
            pid = getattr(self, f"player_{ i }_id")
            if pid is None:
                pl.append(VIERGEWINNT_PLAYER_EMOJIS[i]+" BOT")
            else:
                pl.append(VIERGEWINNT_PLAYER_EMOJIS[i]+" <@"+pid+">")
        return "\n".join(pl)

    def _get_game_info(self):
        if self.finished:
            if self.winner_id:
                if self.winner_id == "BOT":
                    return "Das Spiel ist beendet! Du hast leider verloren!"
                else:
                    return f"Das Spiel ist beendet! Gewonnen hat <@{self.winner_id}>!"
            return "Das Spiel ist beendet! Unentschieden!"
        else:
            p = getattr(self, f"player_{ self.current_player }_id")
            if p is None:
                return f"{ VIERGEWINNT_PLAYER_EMOJIS[self.current_player] } Ich bin an der Reihe! (Berchne einen guten Zug...)"
            else:
                return f"{ VIERGEWINNT_PLAYER_EMOJIS[self.current_player] } <@{ p }> ist an der Reihe!"

    def get_description(self):
        return self._get_game_info()+"\n\n"+self._get_players()+"\n\n"+self._get_gameboard()

    # Get

    @property
    def rows(self):
        return self.game

    @property
    def cols(self):
        return [[self.rows[h][w] for h in range(self.height)] for w in range(self.width)]

    @property
    def dias(self):
        dias = []
        w = 0 
        h = self.height-1
        while w < self.width:
            dia = []
            _w, _h = w, h

            while _w < self.width and _h < self.height:
                dia.append(self.rows[_h][_w])
                _w += 1
                _h += 1

            dias.append(dia)
            if h > 0:
                h -= 1
            else:
                w += 1
        w = 0
        h = 0
        while w < self.width:
            dia = []
            _w, _h = w, h

            while _w < self.width and _h >= 0:
                dia.append(self.rows[_h][_w])
                _w += 1
                _h -= 1

            dias.append(dia)
            if h < self.height-1:
                h += 1
            else:
                w += 1
        return dias

    # Functions

    def _next_player(self):
        self.current_player = (self.current_player+1 if self.current_player < 2 else 1)

    def _can_add_to_column(self, n: int):
        if n >= self.width:
            return None
        return not self.rows[0][n]

    def _add_to_column(self, n:int):
        if self._can_add_to_column(n):
            for h in range(self.height-1, -1, -1):
                if not self.game[h][n]:
                    self.rows[h][n] = self.current_player
                    self._next_player()
                    return True
        else:
            return False

    # Checks

    def _is_full(self):
        return not 0 in self.rows[0]

    def _get_winner(self):
        for ls in [self.rows, self.cols, self.dias]:
            for l in ls:
                for i in range(len(l)-3):
                    if (not l[i] == 0) and (l[i] == l[i+1]) and (l[i+1] == l[i+2]) and (l[i+2] == l[i+3]):
                        return l[i]
        return 0

    # Process Input

    def process(self, col:int, playerid):
        if not (self.finished or self.winner_id):
            if getattr(self, "player_"+str(self.current_player)+"_id", None) == str(playerid):
                if self._add_to_column(col):
                    w = self._get_winner()
                    if w:
                        self.winner_id = getattr(self, "player_"+str(w)+"_id", None)
                        self.finished = True
                    elif self._is_full():
                        self.finished = True
                    return True
        return False

    def process_bot(self):
        if not self.finished and getattr(self, f"player_{self.current_player}_id") is None: 
            if self._add_to_column(VierGewinntBot.get_best_move(board=self.game, botnr=self.current_player, maxdepth=4)):
                w = self._get_winner()
                if w:
                    self.winner_id = "BOT"
                    self.finished = True
                elif self._is_full():
                    self.finished = True
                return True
        return False

    # Meta

    class Meta:
        verbose_name = 'VierGewinnt Game'
        verbose_name_plural = 'VierGewinnt Games'

    objects = models.Manager()



from discordbot.botevents.on_command_error import setup as setup_on_command_error
from discordbot.botevents.on_command import setup as setup_on_command

def setup(bot):
    setup_on_command_error(bot)
    setup_on_command(bot)



from discord.ext import commands
from discord import Embed, Member, User
import time

class Support(commands.Cog):
    def __init__(self, bot):
        self.bot = bot
        self.color = 0xffdf00

    @commands.command(
        brief='Melde einen Spieler',
        description='Benutze diesen Command um Spieler zu melden',
        aliases=[],
        help="Wenn ein Benutzer Blödsinn treibt, dann benutze /report <Member> [Grund]",
        usage="<Member> [Grund]"
        )
    @commands.guild_only()
    async def report(self, ctx, member: Member, *args):
        Grund = " ".join(args)
        Grund = Grund if Grund.rstrip(" ") else "Leer"
        await ctx.database.createReport(dc_user=member, reason=Grund)
        await ctx.sendEmbed(title="Benutzer Gemeldet", color=self.color, fields=[("Betroffener",member.mention),("Grund",Grund)])
        return


    @commands.command(
        brief='Erhalte alle Reports',
        description='Benutze diesen Command um alle Reports zu sehen',
        aliases=["getreports","getreport"],
        help="Mit /getreports [Member] kannst du alle Reports ansehen.",
        usage="[Member]"
        )
    @commands.has_any_role("Moderator","Supporter","Admin")
    @commands.guild_only()
    async def reports(self, ctx, member:Member=None):
        if member == None:
            EMBED = Embed(title="Server Reports", color=self.color)
            EMBED.set_footer(text=f'Angefordert von {ctx.author.name}',icon_url=ctx.author.avatar_url)
            for user in await ctx.database.getReports():
                EMBED.add_field(**user)
            await ctx.send(embed=EMBED)
        else:
            EMBED = Embed(title="User Reports", color=self.color, description=("User: "+member.mention))
            EMBED.set_footer(text=f'Angefordert von {ctx.author.name}',icon_url=ctx.author.avatar_url)
            for report in await ctx.database.getReports(dc_user=member):
                EMBED.add_field(**report)
            await ctx.send(embed=EMBED)
        return




def setup(bot):
    bot.add_cog(Support(bot))



# pylint: disable=unused-variable

from discord.ext import commands
from discord import Embed

def setup(bot):
    @bot.event
    async def on_command_error(ctx, error):
        EMBED = Embed(title="Fehler", color=0xff0000)
        EMBED.set_footer(text=f'Angefordert von {ctx.message.author.name}#{ctx.message.author.discriminator}',icon_url=ctx.author.avatar_url)
        if isinstance(error, commands.BadArgument):
            EMBED.add_field(name="Beschreibung", value="Du hast ungültige Argumente angegeben!")
        elif isinstance(error, commands.MissingRequiredArgument):
            EMBED.add_field(name="Beschreibung", value="Du hast ein benötigtes Argument weggelassen!")
        elif isinstance(error, commands.CommandNotFound):
            EMBED.add_field(name="Beschreibung", value="Dieser Befehl wurde nicht gefunden!")
        elif isinstance(error, commands.CommandError):
            EMBED.add_field(name="Beschreibung", value="Bei einem Befehl ist ein Fehler aufgetreten!")
        elif isinstance(error, commands.CommandOnCooldown):
            EMBED.add_field(name="Beschreibung", value="Warte, bis du diesen Befehl erneut benutzen kannst!")
        elif isinstance(error, commands.DisabledCommand):
            EMBED.add_field(name="Beschreibung", value="Dieser Command ist aktuell deaktiviert!")
        elif isinstance(error, commands.TooManyArguments):
            EMBED.add_field(name="Beschreibung", value="Du hast zu viele Argumente angegeben!")
        elif isinstance(error, commands.MissingPermissions):
            EMBED.add_field(name="Beschreibung", value="Du hast nicht die nötigen Berechtigungen für diesen Command!")
        elif isinstance(error, commands.BotMissingPermissions):
            EMBED.add_field(name="Beschreibung", value="Ich habe nicht die nötigen Berechtigungen für diesen Command!")
        elif isinstance(error, commands.NoPrivateMessage):
            EMBED.add_field(name="Beschreibung", value="Dieser Command kann nur auf einem Server verwendet werden!")
        elif isinstance(error, commands.PrivateMessageOnly):
            EMBED.add_field(name="Beschreibung", value="Dieser Command kann nur in Direktnachrichten werden!")
        elif isinstance(error, commands.MissingRole):
            EMBED.add_field(name="Beschreibung", value="Du hast die benötigten Rollen nicht, um diesen Befehl auszuführen!")
        elif isinstance(error, commands.MissingAnyRole):
            EMBED.add_field(name="Beschreibung", value="Du hast die benötigten Rollen nicht, um diesen Befehl auszuführen!")
        elif isinstance(error, commands.NotOwner):
            EMBED.add_field(name="Beschreibung", value="Du bist nicht Besitzer des Bots!")
            return # keine Nachricht senden!
        else:
            EMBED.add_field(name="Beschreibung", value="Es ist ein unbekannter Fehler aufgetreten! Vermutlich liegt er nicht bei dir, also melde ihn am besten einen Admin.")
            print("[Command] - Bei '"+ctx.message.content+"' von '"+ctx.message.author.name+"#"+ctx.message.author.discriminator+"' ist ein Fehler aufgetreten: "+str(error))
        
        if not error == "":
            EMBED.add_field(name="Text", value=str(error) if len(str(error)) < 1024 else str(error)[-1024:-1])
        EMBED.add_field(name="Nachricht", value=ctx.message.content, inline=False)
        await ctx.send(embed=EMBED)

        if type(error) in [commands.CommandError]:
            raise error



from discord.ext import commands
from discord import Embed
import random

class Help(commands.Cog):
    def __init__(self, bot):
        self.bot = bot
        self.color = 0x000000

    @commands.command(
        name='help',
        brief='Erhalte Hilfe zu Commands',
        aliases=["hilfe", "commands", "command", "?", "cmds"],
        help="Benutze diesen Command für genauere Hilfe zur jeweiligen Kategorie / zum jeweiligen Befehl/Unterbefehl",
        usage="[Kategorie/Befehl/Befehl Unterbefehl]"
    )
    async def help(self, ctx, search:str='*', subsearch:str=None):
        help_embed = ctx.getEmbed(title='Hilfe', color=self.color, inline=False)
        help_embed.description = "Syntax: `/help [Kategorie/Befehl/Befehl Unterbefehl]`\n\n"

        def addCog(cog, include_subcommands=False, hide_cogs=["Owneronly"]):
            if not cog.qualified_name in hide_cogs:
                commands_list = ''
                for cmd in cog.walk_commands():
                    if include_subcommands:
                        depth = 0
                        currentcmd = cmd
                        while currentcmd.parent:
                            depth += 1
                            currentcmd = currentcmd.parent 

                        commands_list += "- "*depth + f'**{cmd.name}** - *{cmd.brief}*\n'
                    elif not cmd.parent:
                        commands_list += f'**{cmd.name}** - *{cmd.brief}*\n'

                help_embed.add_field(
                    name=cog.qualified_name,
                    value=commands_list+'\u200b',
                    inline=False
                )

        def addGroup(grp):
            commands_list = ''
            for cmd in grp.commands:
                commands_list += f'**{cmd.name}** - *{cmd.brief}*\n'

            help_embed.add_field(
                name="Unterbefehle von "+grp.qualified_name,
                value=commands_list+'\u200b',
                inline=False
            )

        def addCommand(cmd):
            if not cmd.hidden:
                if isinstance(cmd, commands.Group):
                    if subsearch:
                        subcmds = dict((alias, cmd) for cmd in cmd.commands for alias in cmd.aliases+[cmd.name.lower()])
                        if subsearch in subcmds:
                            addCommand(subcmds[subsearch])
                            return
                    addGroup(cmd)

                cmdname = cmd.name
                currentcmd = cmd
                while currentcmd.parent:
                    currentcmd = currentcmd.parent
                    cmdname = currentcmd.name + " " + cmdname

                cmdbrief = " - "+cmd.brief if cmd.brief else ""
                help_text = f'```/{cmdname}{cmdbrief}```\n' 

                if cmd.description:
                    help_text += f'Beschrieb: `{cmd.description}`\n\n'
                if cmd.help:
                    help_text += f'Hilfe: `{cmd.help}`\n\n'
                if cmd.aliases:
                    help_text += f'Alias: `{"`, `".join(cmd.aliases)}`'

                help_text += f'\nSyntax: `/{cmdname}{" "+cmd.usage if cmd.usage is not None else ""}`\n\u200b'
                help_embed.description += help_text
    

        cogs = dict((k.lower(), v) for k, v in dict(self.bot.cogs).items())
        cmds = dict((alias, cmd) for cmd in self.bot.commands for alias in cmd.aliases+[cmd.name.lower()])

        if search == '*':
            for cog in cogs:
                addCog(cogs[cog])

        elif search in cmds:
            addCommand(cmds[search])

        elif search in cogs:
            addCog(cogs[search], include_subcommands=True, hide_cogs=[])

        else:
            raise commands.BadArgument("Ungültige(r) Kategorie/Befehl.\nBenutze den `/help` Befehl um alle Kategorien und Befehle zu sehen.")

        await ctx.send(embed=help_embed)
        return


def setup(bot):
    bot.add_cog(Help(bot))



import time
import os
import youtube_dl
import asyncio

from discord import PCMVolumeTransformer, FFmpegPCMAudio
from discord.ext import commands

from asgiref.sync import sync_to_async

#####

filespath = os.path.join(os.path.dirname(os.path.dirname(os.path.realpath(__file__))), "botfiles")

# Suppress noise about console usage from errors
youtube_dl.utils.bug_reports_message = lambda: ''

color = 0xee00ff

#####

class YouTubePlayer(PCMVolumeTransformer):
    _ytdl = youtube_dl.YoutubeDL({
        'format': 'bestaudio/best',
        'outtmpl': os.path.join(filespath,'youtube','%(extractor)s-%(id)s-%(title)s.%(ext)s'),
        'restrictfilenames': True,
        'noplaylist': True,
        'nocheckcertificate': True,
        'ignoreerrors': True,
        'logtostderr': False,
        'quiet': True,
        'no_warnings': True,
        'default_search': 'auto',
        'ffmpeg_location': filespath,
        'source_address': '0.0.0.0' # bind to ipv4 since ipv6 addresses cause issues sometimes
    })

    _ffmpeg_options = {
        'options': '-vn',
        'executable': os.path.join(filespath,"ffmpeg.exe")
    }

    def __init__(self, filename, *, queue, data, volume=0.5):
        source = FFmpegPCMAudio(filename, **self._ffmpeg_options)

        super().__init__(source, volume)

        data.pop("formats")

        self.queue = queue

        self.data = data

        self.url = data.get('url', '')
        self.link = data.get('webpage_url', self.url)
        self.title = data.get('title', 'Unbekannter Titel')

        self.uploader = data.get('uploader', "")
        self.uploader_url = data.get('uploader_url', "")
        self.thumbnail = data.get('thumbnail', "")
        self.description = data.get('description', "")
        self.duration = int(data.get('duration', 0))

    async def send(self, ctx, status:str="Wird jetzt gespielt..."):
        fields = [("Ansehen/Anhören", "[Hier klicken]("+self.link+")")]
        if self.duration:
            fields.append(("Dauer", str(int(self.duration/60))+"min "+str(int(self.duration%60))+"s"))
        fields.append(("Status", status, False))
        await ctx.sendEmbed(
            title=self.title,
            description=((self.description if len(self.description) < 100 else self.description[0:100]+"...") if isinstance(self.description, str) else "Keine Beschreibung gefunden"),
            color=ctx.cog.color,
            fields=fields,
            thumbnailurl=self.thumbnail if self.thumbnail else None,
            authorname=self.uploader if self.uploader else None,
            authorurl=self.uploader_url if self.uploader_url else None,
        )

    def play(self, ctx):
        if ctx.voice_client and ctx.voice_client.is_playing():
            ctx.voice_client.stop()
        elif ctx.voice_client:
            ctx.voice_client.play(self, after=lambda e: self.queue.playNext(ctx))


    @classmethod
    async def from_url(self, url, *, queue, loop=None, stream=False):
        loop = loop or asyncio.get_event_loop()
        data = await loop.run_in_executor(None, lambda: self._ytdl.extract_info(url, download=not stream))

        if 'entries' in data:
            data = data['entries'][0]

        filename = data['url'] if stream else self._ytdl.prepare_filename(data)
        return [self(queue=queue, filename=filename, data=data)]


class MusicQueue():
    def __init__(self, server):
        self.server = server

        self._players = []

    def addPlayer(self, player):
        self._players.append(player)

    def hasPlayer(self):
        return bool(self._players)

    def playNext(self, ctx):
        if self.hasPlayer() and ctx.voice_client and ctx.voice_client.is_connected():
            player = self._players.pop(0)
            player.play(ctx)
            return player
        else:
            return None

    async def sendNowPlaying(self, ctx):
        if ctx.voice_client and ctx.voice_client.source:
            if isinstance(ctx.voice_client.source, YouTubePlayer):
                await ctx.voice_client.source.send(ctx, status="Wird jetzt gespielt!")
        else:
            raise commands.CommandError("Aktuell wird nichts abgespielt.")

    async def createYoutubePlayer(self, search, loop=None, stream=False):
        players = await YouTubePlayer.from_url(search, queue=self, loop=loop, stream=stream)
        if not stream:
            for player in players:
                self.addPlayer(player)
        return players



class Server():
    _all = {}

    def __init__(self,id):
        self.id = id
        self.musicqueue = MusicQueue(server=self)
        #self.polls = {}

    @classmethod
    def getServer(self, serverid:int):
        if not serverid in self._all:
            self._all[serverid] = Server(serverid)
        return self._all[serverid]




### NEW

from discordbot.models import Server as DB_Server, User as DB_User, Report as DB_Report, Member as DB_Member, AmongUsGame, VierGewinntGame

class DjangoConnection():
    def __init__(self, dc_user, dc_guild):
        self.dc_user = dc_user
        self.dc_guild = dc_guild
        self._db_user = None
        self._db_server = None

    @classmethod
    @sync_to_async
    def fetch_user(self, dc_user):
        if not DB_User.objects.filter(id=str(dc_user.id)).exists():
            user = DB_User.objects.create(id=str(dc_user.id), name=dc_user.name+"#"+dc_user.discriminator)
        else:
            user = DB_User.objects.get(id=str(dc_user.id))
            if not user.name == (dc_user.name+"#"+dc_user.discriminator):
                user.name = (dc_user.name+"#"+dc_user.discriminator)
                user.save()
        return user

    async def get_user(self):
        if self._db_user is None:
            self._db_user = await self.fetch_user(self.dc_user)
        return self._db_user

    @classmethod
    @sync_to_async
    def fetch_server(self, dc_guild):
        if not DB_Server.objects.filter(id=str(dc_guild.id)).exists():
            server = DB_Server.objects.create(id=str(dc_guild.id), name=dc_guild.name)
        else:
            server = DB_Server.objects.get(id=str(dc_guild.id))
            if not server.name == dc_guild.name:
                server.name = dc_guild.name
                server.save()
        return server

    async def get_server(self):
        if self._db_server is None:
            self._db_server = await self.fetch_server(self.dc_guild)
        return self._db_server

    @classmethod
    @sync_to_async
    def _join_server(self, user, server):
        user.joinServer(server)

    # Basic Methods

    @classmethod
    @sync_to_async
    def _save(self, game):
        game.save()

    @classmethod
    @sync_to_async
    def _delete(self, game):
        game.delete()

    # Reports

    @classmethod
    @sync_to_async
    def _createReport(self, **kwargs):
        return DB_Report.objects.create(**kwargs)

    async def createReport(self, dc_user, reason:str=""):
        server = await self.get_server()
        user = await self.get_user()
        await self._join_server(user, server)
        reporteduser = await self.fetch_user(dc_user)
        await self._join_server(reporteduser, server)
        return await self._createReport(server=server, user=reporteduser, reported_by=user, reason=reason)

    @classmethod
    @sync_to_async
    def _getReports(self, server, **kwargs):
        return server.getReports(**kwargs)

    async def getReports(self, dc_user=None, **kwargs):
        server = await self.get_server()
        if dc_user is None:
            return await self._getReports(server, **kwargs)
        else:
            user = await self.fetch_user(dc_user)
            return await self._getReports(server, user=user, **kwargs)

    # AmongUs

    @classmethod
    @sync_to_async
    def _getAmongUsGame(self, **kwargs):
        return AmongUsGame.objects.get(**kwargs)

    async def getAmongUsGame(self, **kwargs):
        user = await self.get_user()
        server = await self.get_server()
        return await self._getAmongUsGame(creator=user, guild=server, **kwargs)

    @classmethod
    @sync_to_async
    def _hasAmongUsGame(self, **kwargs):
        return AmongUsGame.objects.filter(**kwargs).exists()

    async def hasAmongUsGame(self, **kwargs):
        user = await self.get_user()
        server = await self.get_server()
        return await self._hasAmongUsGame(creator=user, guild=server, **kwargs)

    @classmethod
    @sync_to_async
    def _createAmongUsGame(self, **kwargs):
        return AmongUsGame.objects.create(**kwargs)

    async def createAmongUsGame(self, **kwargs):
        user = await self.get_user()
        server = await self.get_server()
        return await self._createAmongUsGame(creator=user, guild=server, **kwargs)

    @classmethod
    @sync_to_async
    def _setAmongUsUser(self, game: AmongUsGame, userid: int, color: str, save: bool = False):
        game.set_user(userid=userid, color=color, save=save)

    @classmethod
    @sync_to_async
    def _removeAmongUsUser(self, game: AmongUsGame, userid: int, save: bool = False):
        game.remove_user(userid=userid, save=save)

    # VierGewinnt

    @classmethod
    @sync_to_async
    def _getVierGewinntGame(self, **kwargs):
        return VierGewinntGame.objects.get(**kwargs)

    @classmethod
    @sync_to_async
    def _hasVierGewinntGame(self, **kwargs):
        return VierGewinntGame.objects.filter(**kwargs).exists()

    @classmethod
    @sync_to_async
    def _createVierGewinntGame(self, **kwargs):
        return VierGewinntGame.create(**kwargs)

